"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var utils_1 = require("./../../../utils");
var base_1 = require("./../../base");
var base_2 = require("./../base");
var TsClassBinder = (function (_super) {
    __extends(TsClassBinder, _super);
    function TsClassBinder(factory, node) {
        _super.call(this, new base_2.TsBaseDefinitionBinder(), new base_2.TsNamedBinderByNode(node), new base_2.TsExportableBinder(node), new base_2.TsAmbientableBinder(node), new base_2.TsTypeParameteredBinderByNode(factory, node), new base_2.TsAbstractableBinder(node), new base_2.TsDecoratableBinder(factory, node), new base_2.TsNodedBinder(factory, node));
        this.factory = factory;
        this.node = node;
    }
    TsClassBinder.prototype.getExtendsTypes = function () {
        var _this = this;
        // todo: not the most efficient thing to loop through all the children each time
        return this.node.getChildren()
            .filter(function (node) { return node.isHeritageClause() && node.hasExtendsKeyword(); })
            .map(function (node) { return node.getHeritageTypeNodes(); })
            .reduce(function (a, b) { return a.concat(b); }, [])
            .map(function (typeNode) { return _this.factory.getTypeFromTypeNode(typeNode); });
    };
    TsClassBinder.prototype.getImplementsTypes = function () {
        var _this = this;
        return this.node.getChildren()
            .filter(function (node) { return node.isHeritageClause() && node.hasImplementsKeyword(); })
            .map(function (node) { return node.getHeritageTypeNodes(); })
            .reduce(function (a, b) { return a.concat(b); }, [])
            .map(function (typeNode) { return _this.factory.getTypeFromTypeNode(typeNode); });
    };
    TsClassBinder.prototype.getMembers = function () {
        var _this = this;
        var container = new base_1.ClassMemberContainer();
        var constructors = [];
        // because there can be multiple method signatures, the last one needs to be used
        var methods = {};
        var staticMethods = {};
        // because there can be set and get accessor nodes
        var properties = {};
        this.node.getChildren().forEach(function (childNode) {
            utils_1.tryGet(childNode, function () {
                if (childNode.isMethodDeclaration()) {
                    var key = childNode.getNameKey();
                    if (childNode.hasStaticKeyword()) {
                        if (typeof staticMethods[key] === "undefined")
                            staticMethods[key] = [];
                        staticMethods[key].push(childNode);
                    }
                    else {
                        if (typeof methods[key] === "undefined")
                            methods[key] = [];
                        methods[key].push(childNode);
                    }
                }
                else if (childNode.isPropertyDeclaration() || childNode.isGetAccessor() || childNode.isSetAccessor()) {
                    if (childNode.hasStaticKeyword()) {
                        container.staticProperties.push(_this.factory.getClassStaticProperty(childNode));
                    }
                    else {
                        var key = childNode.getNameKey();
                        if (typeof properties[key] === "undefined")
                            properties[key] = [];
                        properties[key].push(childNode);
                    }
                }
                else if (childNode.isConstructor()) {
                    constructors.push(childNode);
                }
                else if (childNode.isSetAccessor()) {
                }
                else if (childNode.isIdentifier()) {
                }
                else if (childNode.isTypeParameter()) {
                }
                else if (childNode.isExportKeyword()) {
                }
                else if (childNode.isDefaultKeyword()) {
                }
                else if (childNode.isHeritageClause()) {
                }
                else {
                    utils_1.Logger.warn("Unknown class child kind: " + childNode.nodeKindToString());
                }
            });
        });
        if (constructors.length > 0) {
            utils_1.tryGet(constructors[constructors.length - 1], function () {
                container.constructorDef = _this.factory.getClassConstructor(constructors);
            });
        }
        Object.keys(methods).forEach(function (nameKey) {
            var childNodes = methods[nameKey];
            utils_1.tryGet(childNodes[childNodes.length - 1], function () {
                container.methods.push(_this.factory.getClassMethod(childNodes));
            });
        });
        Object.keys(staticMethods).forEach(function (nameKey) {
            var childNodes = staticMethods[nameKey];
            utils_1.tryGet(childNodes[childNodes.length - 1], function () {
                container.staticMethods.push(_this.factory.getClassStaticMethod(childNodes));
            });
        });
        Object.keys(properties).forEach(function (nameKey) {
            var childNodes = properties[nameKey];
            utils_1.tryGet(childNodes[childNodes.length - 1], function () {
                container.properties.push(_this.factory.getClassProperty(childNodes));
            });
        });
        return container;
    };
    return TsClassBinder;
}(base_1.ClassBinder));
exports.TsClassBinder = TsClassBinder;

//# sourceMappingURL=TsClassBinder.js.map
