"use strict";
function __export(m) {
    for (var p in m) /* istanbul ignore else */ if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
var tmp = require("tmp");
var fs = require("fs");
var TsMain_1 = require("./compiler/TsMain");
var errors_1 = require("./errors");
var definitions_1 = require("./definitions");
var factories_1 = require("./factories");
var utils_1 = require("./utils");
__export(require("./definitions"));
__export(require("./errors"));
__export(require("./createFunctions"));
function getInfoFromFiles(fileNames, options) {
    if (!(fileNames instanceof Array)) {
        throw new errors_1.ArgumentTypeError("fileNames", "array");
    }
    options = options || {};
    var _a = options.includeTsNodes, includeTsNodes = _a === void 0 ? false : _a, _b = options.showDebugMessages, showDebugMessages = _b === void 0 ? false : _b;
    utils_1.Logger.toggleEnabled(showDebugMessages);
    var tsMain = new TsMain_1.TsMain(fileNames, options);
    var tsFactory = new factories_1.TsFactory({ includeTsNodes: includeTsNodes });
    var fileDefinitions = tsMain.getSourceFiles().map(function (sourceFile) { return tsFactory.getFileDefinition(sourceFile); });
    tsFactory.bindDeferred();
    tsFactory.fillAllCachedTypesWithDefinitions();
    var globalDef = new definitions_1.GlobalDefinition();
    globalDef.files = fileDefinitions;
    if (includeTsNodes)
        globalDef.typeChecker = tsMain.getTypeScriptTypeChecker();
    return globalDef;
}
exports.getInfoFromFiles = getInfoFromFiles;
function getInfoFromString(code, options) {
    if (typeof code !== "string") {
        throw new errors_1.ArgumentTypeError("code", "string");
    }
    var tmpFile = tmp.fileSync({ postfix: ".ts" });
    var fileDefinition;
    try {
        code = utils_1.StringUtils.ensureEndsWithNewline(code);
        fs.writeFileSync(tmpFile.name, code);
        fileDefinition = getInfoFromFiles([tmpFile.name], options).files[0];
    }
    finally {
        tmpFile.removeCallback();
    }
    return fileDefinition;
}
exports.getInfoFromString = getInfoFromString;

//# sourceMappingURL=main.js.map
